/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project ( https://gitlab.com/desq/ )
	* Qt5 example of analog clock, and Lenovo K8 Note's clock widget
	* are the inspiration for this. Some parts of the code are taken
	* from Qt5's example
	*
	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*
	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*
	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QtWidgets>
#include "BrightnessWidget.hpp"

#include <desq/DesQUtils.hpp>

BacklightDevice::BacklightDevice( QString path ) {

	devPath = QString( path );

	QFile curFile( devPath + "/max_brightness" );

	maxBrightness = 255.0;

	if ( curFile.open( QIODevice::ReadOnly ) )
		maxBrightness = QString::fromLocal8Bit( curFile.readAll() ).simplified().toDouble();

	curFile.close();
};

QString BacklightDevice::name() {

	return QFileInfo( devPath ).fileName();
};

qreal BacklightDevice::currentBrightness() {

	/* Brightness scaled to 1000 */
	QFile curFile( devPath + "/brightness" );

	if ( not curFile.open( QIODevice::ReadOnly ) )
		return -1;

	qreal curBrightness = QString::fromLocal8Bit( curFile.readAll() ).simplified().toDouble();
	curFile.close();

	return curBrightness * 1000.0 / maxBrightness;
};

bool BacklightDevice::setBrightness( qreal value ) {

	QFile curFile( devPath + "/brightness" );

	// if ( not curFile.open( QIODevice::WriteOnly ) ) {
		// CPrime::InfoFunc::messageEngine(
			// "CoreAction",
			// "coreaction",
			// "Brightness Add-on",
			// "We do not seem to have permission to write to this file. Please ensure that the udev rules are installed and enabled.",
			// nullptr
		 // );

		 // qDebug() << "Unable to write to file.";

		// return false;
	// }

	if ( abs( currentBrightness() - value * 1000 ) < 1 )
		return true;

	qreal brightness = 1.0 * value * maxBrightness;

	if ( not brightness )
		brightness++;

	QProcess proc;
	proc.start( "pkexec", { DesQUtils::getUtilityPath( "pkit" ), "--brightness", QString::number( (int)brightness ), devPath + "/brightness" } );
	proc.waitForFinished( -1 );

	return true;
};

BrightnessWidget::BrightnessWidget( QWidget *parent ) : QWidget( parent ) {

	QFile svgf( ":/monitor.svg" );
	svgf.open( QFile::ReadOnly );
	svg = svgf.readAll();
	svgf.close();

	display = new QSvgWidget( this );
	display->load( QByteArray( svg ).replace( QByteArray( "###" ), QByteArray( "0.5" ) ) );
	display->setFixedSize( 100, 100 );

	setFixedSize( 200, 120 );

	QHBoxLayout *lyt = new QHBoxLayout();
	lyt->addWidget( display, Qt::AlignCenter );

	setLayout( lyt );

	opacity = new QGraphicsOpacityEffect( this );
	opacity->setOpacity( 0.4 );
	setGraphicsEffect( opacity );
};

void BrightnessWidget::wheelEvent( QWheelEvent *wEvent ) {

	int angle = wEvent->angleDelta().y() / 8;
	qreal newVal = -1;

    Q_FOREACH( QString dev, QDir( "/sys/class/backlight/" ).entryList( QDir::NoDotAndDotDot | QDir::AllEntries ) ) {
        BacklightDevice device( QFileInfo( "/sys/class/backlight/" + dev ).symLinkTarget() );

        qreal curVal = device.currentBrightness();
        newVal = ( curVal + angle ) / 1000.0;

        if ( newVal <= 0 )
			newVal = 0.01;

		device.setBrightness( newVal );
	}

	if ( newVal > 0 )
		display->load( QByteArray( svg ).replace( QByteArray( "###" ), QByteArray::number( newVal, 'f', 2 ) ) );

	wEvent->accept();
};

void BrightnessWidget::enterEvent( QEvent *event ) {

	opacity->setOpacity( 1.0 );
	event->accept();
};

void BrightnessWidget::leaveEvent( QEvent *event ) {

	opacity->setOpacity( 0.4 );
	event->accept();
};
